/*
 * Package : mraa
 * Author : S. Hamblett <steve.hamblett@linux.com>
 * Date   : 03/10/2019
 * Copyright :  S.Hamblett
 */

import 'dart:io';

// ignore_for_file: omit_local_variable_types
// ignore_for_file: unnecessary_final
// ignore_for_file: cascade_invocations
// ignore_for_file: avoid_print

/// Platform support helper class for testing
class MraaPlatformHelper {
  /// Construction
  MraaPlatformHelper() {
    _loadOsDetails();
  }

  final Map<String, String> _osMap = <String, String>{};
  Map<String, String> get osMap => _osMap;

  String _distName = 'None';

  /// Is this a Centos distro
  bool isCentos() => _distName.toLowerCase().contains('centos');

  /// Is this a Fedora distro
  bool isFedora() => _distName.toLowerCase().contains('fedora');

  /// Is this an Ubuntu  distro
  bool isUbuntu() => _distName.toLowerCase().contains('ubunto');

  /// Get the correct test library
  String getTestLib() {
    if (isCentos()) {
      return 'test/testlib/centos7/libmraa.so.2.0.0';
    }
    if (isFedora()) {
      return 'test/testlib/fedora/libmraa.so.2.0.0';
    }
    // Default to Ubuntu for Travis
    return 'test/testlib/ubuntu/libmraa.so.2.0.0';
  }

  void _loadOsDetails() {
    const String osFilePath = '/etc/os-release';
    final File osFile = File(osFilePath);
    final List<String> lines = osFile.readAsLinesSync();
    for (final String line in lines) {
      if (line.isEmpty) {
        continue;
      }
      final List<String> vals = line.split('=');
      vals[1] = vals[1].replaceAll('"', '');
      _osMap[vals[0]] = vals[1];
    }
    // Set the dist name
    _distName = _osMap['NAME'];
  }

  @override
  String toString() => _osMap.toString();
}
